#!/usr/bin/env bash

SCRIPT_PATH=$(cd $(dirname "$0")/ && pwd)
ROOT_DIR=$(dirname "$SCRIPT_PATH")

# OS to build on
DOCKER_OS=centos
DEFAULT_CENTOS_VERSION=centos7.9.2009
DEFAULT_UBUNTU_VERSION=18.04

CMA_SERVER_PROFILE=prod
AM2CM_DOCKER_IMAGE_VERSION=latest

AM2CM_VENV=venv

for i in "$@"; do
  case $i in
  -os=* | --docker-os=*)
    DOCKER_OS=${i#*=}
    ;;
  -iv=* | --image-version=*)
    AM2CM_DOCKER_IMAGE_VERSION=${i#*=}
    ;;
  -p=* | --profile=*)
    CMA_SERVER_PROFILE=${i#*=}
    ;;
  -d | --dev-mode)
    DEV_MODE=true
    ;;
  -h | --help)
    HELP=true
    ;;
  *)
    echo "Unknown option: $i"
    HELP=true
    ;;
  esac
done

__usage="
 Usage: $(basename "$0") [options]
 Options:
    -os, --docker-os: OS to build on
    -iv, --image-version: Set the version tag for the docker image
    -p, --profile: Set the active profile(s) for the cma-server
    -h, --help: Print this help
 "

if [ "${HELP}" = true ]; then
  echo "$__usage"
  exit 0
fi

if [ -z "${DOCKER_OS}" ]; then
  echo "ERROR: The OS for docker container is not specified!"
  exit 1;
fi

if [ -z "${CMA_SERVER_PROFILE}" ]; then
  echo "ERROR: The profile for the cma-server is not specified!"
  exit 1;
fi

case ${DOCKER_OS} in
  "centos") DOCKER_OS_VERSION=$DEFAULT_CENTOS_VERSION ;;
  "ubuntu") DOCKER_OS_VERSION=$DEFAULT_UBUNTU_VERSION ;;
   *)
     echo "ERROR: ${DOCKER_OS} is unsupported!!"
     exit 1
     ;;
esac

AM2CM_DOCKER_IMAGE=cma-"${DOCKER_OS}"


AM2CM_ROOT_DIR=/$(basename "${ROOT_DIR}")

SRC_CMA_SERVER=cma-server
SRC_AM2CM_ANSIBLE=am2cm-ansible
SRC_AM2CM_TOOL=am2cm-tool
SRC_AM2CM_UPGRADE=am2cm-upgrade
SRC_AM2CM_DIFF=am2cm-diff
SRC_AM2CM_SOLR_CLIENT=am2cm-solr-client
SRC_CMA_PVC_ANSIBLE=cma-pvc-ansible
SRC_STARTUP_SCRIPT=docker/startup.sh

if [ "${DEV_MODE}" = true ]; then
  SRC_CMA_SERVER=$SRC_CMA_SERVER/target/$SRC_CMA_SERVER
  SRC_AM2CM_ANSIBLE=$SRC_AM2CM_ANSIBLE/target/$SRC_AM2CM_ANSIBLE
  SRC_AM2CM_TOOL=$SRC_AM2CM_TOOL/target/$SRC_AM2CM_TOOL
  SRC_AM2CM_UPGRADE=am2cm-upgrade-tool/target/$SRC_AM2CM_UPGRADE
  SRC_AM2CM_DIFF=am2cm-diff/target/$SRC_AM2CM_DIFF
  SRC_AM2CM_SOLR_CLIENT=am2cm-solr-client/target/$SRC_AM2CM_SOLR_CLIENT
  SRC_CMA_PVC_ANSIBLE=cma-pvc-ansible/target/cma-pvc-ansible
fi

# It's important to add the docker profile to the end of the active profiles!!!
CMA_SERVER_PROFILE=${CMA_SERVER_PROFILE},docker

echo "Calling docker build command..."
docker build -t "${AM2CM_DOCKER_IMAGE}":"${AM2CM_DOCKER_IMAGE_VERSION}" \
  --build-arg DOCKER_OS="${DOCKER_OS}" --build-arg DOCKER_OS_VERSION="${DOCKER_OS_VERSION}" \
  --build-arg AM2CM_ROOT_DIR="${AM2CM_ROOT_DIR}"  --build-arg CMA_SERVER_PROFILE="${CMA_SERVER_PROFILE}" \
  --build-arg SRC_CMA_SERVER=${SRC_CMA_SERVER} --build-arg SRC_AM2CM_ANSIBLE=${SRC_AM2CM_ANSIBLE} \
  --build-arg SRC_AM2CM_TOOL=${SRC_AM2CM_TOOL} --build-arg SRC_AM2CM_UPGRADE=${SRC_AM2CM_UPGRADE} \
  --build-arg SRC_AM2CM_DIFF=${SRC_AM2CM_DIFF} --build-arg SRC_AM2CM_SOLR_CLIENT=${SRC_AM2CM_SOLR_CLIENT} \
  --build-arg SRC_CMA_PVC_ANSIBLE=${SRC_CMA_PVC_ANSIBLE} --build-arg AM2CM_VENV=${AM2CM_VENV} \
  --build-arg SRC_STARTUP_SCRIPT="${SRC_STARTUP_SCRIPT}" \
  -f "${ROOT_DIR}"/docker/Dockerfile "${ROOT_DIR}" \
  --target am2cm-final

if [ $? -ne 0 ]; then
  echo "Docker build command failed... Exiting..."
  exit 1
fi

docker tag "${AM2CM_DOCKER_IMAGE}":"${AM2CM_DOCKER_IMAGE_VERSION}" "${AM2CM_DOCKER_IMAGE}":latest

if [ $? -ne 0 ]; then
  echo "Docker tag command failed... Exiting..."
  exit 1
fi
